/*
 * $Id$
 */

package org.codehaus.mojo.appassembler.model.io.stax;

  //---------------------------------/
 //- Imported classes and packages -/
//---------------------------------/

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.text.DateFormat;
import java.util.Iterator;
import java.util.Locale;
import java.util.jar.Manifest;
import javanet.staxutils.IndentingXMLStreamWriter;
import javax.xml.stream.*;
import org.codehaus.mojo.appassembler.model.Classpath;
import org.codehaus.mojo.appassembler.model.ClasspathElement;
import org.codehaus.mojo.appassembler.model.Daemon;
import org.codehaus.mojo.appassembler.model.Dependency;
import org.codehaus.mojo.appassembler.model.Directory;
import org.codehaus.mojo.appassembler.model.GeneratorConfiguration;
import org.codehaus.mojo.appassembler.model.JvmSettings;
import org.codehaus.plexus.util.xml.Xpp3Dom;

/**
 * Class AppassemblerModelStaxWriter.
 * 
 * @version $Revision$ $Date$
 */
public class AppassemblerModelStaxWriter {


      //--------------------------/
     //- Class/Member Variables -/
    //--------------------------/

    /**
     * Field curId.
     */
    private int curId;

    /**
     * Field idMap.
     */
    private java.util.Map idMap;


      //----------------/
     //- Constructors -/
    //----------------/

    public AppassemblerModelStaxWriter() {
        idMap = new java.util.HashMap();
    } //-- org.codehaus.mojo.appassembler.model.io.stax.AppassemblerModelStaxWriter()


      //-----------/
     //- Methods -/
    //-----------/

    /**
     * Method write.
     * 
     * @param writer
     * @param daemon
     * @throws java.io.IOException
     * @throws XMLStreamException
     */
    public void write( Writer writer, Daemon daemon )
        throws java.io.IOException, XMLStreamException
    {
        XMLOutputFactory factory = XMLOutputFactory.newInstance();
        boolean supportWindowsLineEndings = false;
        if ( factory.isPropertySupported( "com.ctc.wstx.outputEscapeCr" ) )
        {
            factory.setProperty( "com.ctc.wstx.outputEscapeCr", Boolean.FALSE );
            supportWindowsLineEndings = true;
        }
        if ( factory.isPropertySupported( "org.codehaus.stax2.automaticEmptyElements" ) )
        {
            factory.setProperty( "org.codehaus.stax2.automaticEmptyElements", Boolean.FALSE );
        }
        IndentingXMLStreamWriter serializer = new IndentingXMLStreamWriter( factory.createXMLStreamWriter( writer ) );
        if ( supportWindowsLineEndings )
        {
            serializer.setNewLine( serializer.getLineSeparator() );
        }
        serializer.writeStartDocument( daemon.getModelEncoding(), "1.0" );
        writeDaemon( daemon, "daemon", serializer );
        serializer.writeEndDocument();
    } //-- void write( Writer, Daemon ) 

    /**
     * Method writeClasspath.
     * 
     * @param classpath
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     * @throws XMLStreamException
     */
    private void writeClasspath( Classpath classpath, String tagName, XMLStreamWriter serializer )
        throws java.io.IOException, XMLStreamException
    {
        if ( classpath != null )
        {
            serializer.writeStartElement( tagName );
            if ( classpath.getDependencies() != null && classpath.getDependencies().size() > 0 )
            {
                serializer.writeStartElement( "dependencies" );
                for ( Iterator iter = classpath.getDependencies().iterator(); iter.hasNext(); )
                {
                    Dependency o = (Dependency) iter.next();
                    writeDependency( o, "dependency", serializer );
                }
                serializer.writeEndElement();
            }
            if ( classpath.getDirectories() != null && classpath.getDirectories().size() > 0 )
            {
                serializer.writeStartElement( "directories" );
                for ( Iterator iter = classpath.getDirectories().iterator(); iter.hasNext(); )
                {
                    Directory o = (Directory) iter.next();
                    writeDirectory( o, "directory", serializer );
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeClasspath( Classpath, String, XMLStreamWriter ) 

    /**
     * Method writeClasspathElement.
     * 
     * @param classpathElement
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     * @throws XMLStreamException
     */
    private void writeClasspathElement( ClasspathElement classpathElement, String tagName, XMLStreamWriter serializer )
        throws java.io.IOException, XMLStreamException
    {
        if ( classpathElement != null )
        {
            serializer.writeStartElement( tagName );
            if ( classpathElement.getRelativePath() != null )
            {
                serializer.writeStartElement( "relativePath" );
                serializer.writeCharacters( classpathElement.getRelativePath() );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeClasspathElement( ClasspathElement, String, XMLStreamWriter ) 

    /**
     * Method writeDaemon.
     * 
     * @param daemon
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     * @throws XMLStreamException
     */
    private void writeDaemon( Daemon daemon, String tagName, XMLStreamWriter serializer )
        throws java.io.IOException, XMLStreamException
    {
        if ( daemon != null )
        {
            serializer.writeStartElement( tagName );
            if ( daemon.getId() != null )
            {
                serializer.writeStartElement( "id" );
                serializer.writeCharacters( daemon.getId() );
                serializer.writeEndElement();
            }
            if ( daemon.getMainClass() != null )
            {
                serializer.writeStartElement( "mainClass" );
                serializer.writeCharacters( daemon.getMainClass() );
                serializer.writeEndElement();
            }
            if ( daemon.getClasspath() != null )
            {
                writeClasspath( (Classpath) daemon.getClasspath(), "classpath", serializer );
            }
            if ( daemon.getCommandLineArguments() != null && daemon.getCommandLineArguments().size() > 0 )
            {
                serializer.writeStartElement( "commandLineArguments" );
                for ( Iterator iter = daemon.getCommandLineArguments().iterator(); iter.hasNext(); )
                {
                    String commandLineArgument = (String) iter.next();
                    serializer.writeStartElement( "commandLineArgument" );
                    serializer.writeCharacters( commandLineArgument );
                    serializer.writeEndElement();
                }
                serializer.writeEndElement();
            }
            if ( daemon.getJvmSettings() != null )
            {
                writeJvmSettings( (JvmSettings) daemon.getJvmSettings(), "jvmSettings", serializer );
            }
            if ( daemon.getEnvironmentSetupFileName() != null )
            {
                serializer.writeStartElement( "environmentSetupFileName" );
                serializer.writeCharacters( daemon.getEnvironmentSetupFileName() );
                serializer.writeEndElement();
            }
            if ( daemon.isShowConsoleWindow() != true )
            {
                serializer.writeStartElement( "showConsoleWindow" );
                serializer.writeCharacters( String.valueOf( daemon.isShowConsoleWindow() ) );
                serializer.writeEndElement();
            }
            if ( daemon.getGeneratorConfigurations() != null && daemon.getGeneratorConfigurations().size() > 0 )
            {
                serializer.writeStartElement( "generatorConfigurations" );
                for ( Iterator iter = daemon.getGeneratorConfigurations().iterator(); iter.hasNext(); )
                {
                    GeneratorConfiguration o = (GeneratorConfiguration) iter.next();
                    writeGeneratorConfiguration( o, "generatorConfiguration", serializer );
                }
                serializer.writeEndElement();
            }
            if ( daemon.getRepositoryName() != null && !daemon.getRepositoryName().equals( "repo" ) )
            {
                serializer.writeStartElement( "repositoryName" );
                serializer.writeCharacters( daemon.getRepositoryName() );
                serializer.writeEndElement();
            }
            if ( daemon.getLicenseHeaderFile() != null )
            {
                serializer.writeStartElement( "licenseHeaderFile" );
                serializer.writeCharacters( daemon.getLicenseHeaderFile() );
                serializer.writeEndElement();
            }
            if ( daemon.getWrapperLogFile() != null )
            {
                serializer.writeStartElement( "wrapperLogFile" );
                serializer.writeCharacters( daemon.getWrapperLogFile() );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeDaemon( Daemon, String, XMLStreamWriter ) 

    /**
     * Method writeDependency.
     * 
     * @param dependency
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     * @throws XMLStreamException
     */
    private void writeDependency( Dependency dependency, String tagName, XMLStreamWriter serializer )
        throws java.io.IOException, XMLStreamException
    {
        if ( dependency != null )
        {
            serializer.writeStartElement( tagName );
            if ( dependency.getGroupId() != null )
            {
                serializer.writeStartElement( "groupId" );
                serializer.writeCharacters( dependency.getGroupId() );
                serializer.writeEndElement();
            }
            if ( dependency.getArtifactId() != null )
            {
                serializer.writeStartElement( "artifactId" );
                serializer.writeCharacters( dependency.getArtifactId() );
                serializer.writeEndElement();
            }
            if ( dependency.getVersion() != null )
            {
                serializer.writeStartElement( "version" );
                serializer.writeCharacters( dependency.getVersion() );
                serializer.writeEndElement();
            }
            if ( dependency.getClassifier() != null )
            {
                serializer.writeStartElement( "classifier" );
                serializer.writeCharacters( dependency.getClassifier() );
                serializer.writeEndElement();
            }
            if ( dependency.getRelativePath() != null )
            {
                serializer.writeStartElement( "relativePath" );
                serializer.writeCharacters( dependency.getRelativePath() );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeDependency( Dependency, String, XMLStreamWriter ) 

    /**
     * Method writeDirectory.
     * 
     * @param directory
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     * @throws XMLStreamException
     */
    private void writeDirectory( Directory directory, String tagName, XMLStreamWriter serializer )
        throws java.io.IOException, XMLStreamException
    {
        if ( directory != null )
        {
            serializer.writeStartElement( tagName );
            if ( directory.getRelativePath() != null )
            {
                serializer.writeStartElement( "relativePath" );
                serializer.writeCharacters( directory.getRelativePath() );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeDirectory( Directory, String, XMLStreamWriter ) 

    /**
     * Method writeDom.
     * 
     * @param dom
     * @param serializer
     * @throws XMLStreamException
     */
    public void writeDom( Xpp3Dom dom, XMLStreamWriter serializer )
        throws XMLStreamException
    {
        serializer.writeStartElement( dom.getName() );
        String[] attributeNames = dom.getAttributeNames();
        for ( int i = 0; i < attributeNames.length; i++ )
        {
            String attributeName = attributeNames[i];
            serializer.writeAttribute( attributeName, dom.getAttribute( attributeName ) );
        }
        Xpp3Dom[] children = dom.getChildren();
        for ( int i = 0; i < children.length; i++ )
        {
            writeDom( children[i], serializer );
        }
        String value = dom.getValue();
        if ( value != null )
        {
            serializer.writeCharacters( value );
        }
        serializer.writeEndElement();
    } //-- void writeDom( Xpp3Dom, XMLStreamWriter ) 

    /**
     * Method writeGeneratorConfiguration.
     * 
     * @param generatorConfiguration
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     * @throws XMLStreamException
     */
    private void writeGeneratorConfiguration( GeneratorConfiguration generatorConfiguration, String tagName, XMLStreamWriter serializer )
        throws java.io.IOException, XMLStreamException
    {
        if ( generatorConfiguration != null )
        {
            serializer.writeStartElement( tagName );
            if ( generatorConfiguration.getGenerator() != null )
            {
                serializer.writeStartElement( "generator" );
                serializer.writeCharacters( generatorConfiguration.getGenerator() );
                serializer.writeEndElement();
            }
            if ( generatorConfiguration.getConfiguration() != null && generatorConfiguration.getConfiguration().size() > 0 )
            {
                serializer.writeStartElement( "configuration" );
                for ( Iterator iter = generatorConfiguration.getConfiguration().keySet().iterator(); iter.hasNext(); )
                {
                    String key = (String) iter.next();
                    String value = (String) generatorConfiguration.getConfiguration().get( key );
                    serializer.writeStartElement( "" + key + "" );
                    serializer.writeCharacters( value );
                    serializer.writeEndElement();
                }
                serializer.writeEndElement();
            }
            if ( generatorConfiguration.getIncludes() != null && generatorConfiguration.getIncludes().size() > 0 )
            {
                serializer.writeStartElement( "includes" );
                for ( Iterator iter = generatorConfiguration.getIncludes().iterator(); iter.hasNext(); )
                {
                    String include = (String) iter.next();
                    serializer.writeStartElement( "include" );
                    serializer.writeCharacters( include );
                    serializer.writeEndElement();
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeGeneratorConfiguration( GeneratorConfiguration, String, XMLStreamWriter ) 

    /**
     * Method writeJvmSettings.
     * 
     * @param jvmSettings
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     * @throws XMLStreamException
     */
    private void writeJvmSettings( JvmSettings jvmSettings, String tagName, XMLStreamWriter serializer )
        throws java.io.IOException, XMLStreamException
    {
        if ( jvmSettings != null )
        {
            serializer.writeStartElement( tagName );
            if ( jvmSettings.getInitialMemorySize() != null )
            {
                serializer.writeStartElement( "initialMemorySize" );
                serializer.writeCharacters( jvmSettings.getInitialMemorySize() );
                serializer.writeEndElement();
            }
            if ( jvmSettings.getMaxMemorySize() != null )
            {
                serializer.writeStartElement( "maxMemorySize" );
                serializer.writeCharacters( jvmSettings.getMaxMemorySize() );
                serializer.writeEndElement();
            }
            if ( jvmSettings.getMaxStackSize() != null )
            {
                serializer.writeStartElement( "maxStackSize" );
                serializer.writeCharacters( jvmSettings.getMaxStackSize() );
                serializer.writeEndElement();
            }
            if ( jvmSettings.getSystemProperties() != null && jvmSettings.getSystemProperties().size() > 0 )
            {
                serializer.writeStartElement( "systemProperties" );
                for ( Iterator iter = jvmSettings.getSystemProperties().iterator(); iter.hasNext(); )
                {
                    String systemProperty = (String) iter.next();
                    serializer.writeStartElement( "systemProperty" );
                    serializer.writeCharacters( systemProperty );
                    serializer.writeEndElement();
                }
                serializer.writeEndElement();
            }
            if ( jvmSettings.getExtraArguments() != null && jvmSettings.getExtraArguments().size() > 0 )
            {
                serializer.writeStartElement( "extraArguments" );
                for ( Iterator iter = jvmSettings.getExtraArguments().iterator(); iter.hasNext(); )
                {
                    String extraArgument = (String) iter.next();
                    serializer.writeStartElement( "extraArgument" );
                    serializer.writeCharacters( extraArgument );
                    serializer.writeEndElement();
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeJvmSettings( JvmSettings, String, XMLStreamWriter ) 


}
